/*******************************************************************************
Copyright © 2023 Sebastian Roith

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*******************************************************************************/

(function(window) {

const LEVELS = function() {
    const template = {
        task: 'Wähle die Welt aus, in der das Programm nicht zu einer Notabschaltung führt. Starte dort den Roboter.',
        readonly: true,
        locked: true,
        checkCode: [
            CodeCheckers.accept()
        ]
    };
    return [
        {
            code: 'SOLANGE istBelegt\n    vor\nvor',
            worlds: [
                '05120000012',
                '0513000001022',
                '0513010101022',
                '05150000010201042',
                '051100002',
                '051103002'
            ]
        },
        {
            code: 'SOLANGE istNorden\n    WIEDERHOLE 4 MAL\n        linksrum\nvor\nminus',
            worlds: [
                '0533000100020004000800111',
                '05120001000',
                '0513000001010',
                '05120001012'
            ]
        },
        {
            code: 'SOLANGE istBegehbar\n    vor\n    vor\n    vor\nminus',
            worlds: [
                '05120100003',
                '0513000001000',
                '05120001000',
                '051301000000',
                '0512010000',
                '05150101010101000'
            ]
        }
    ].map(({ code, worlds, task, problem }) => {
        const level = Object.create(template);
        if (task)
            level.task = task;
        level.code = code;
        level.world = worlds
            .map(world => ({ world, random: Math.random() }))
            .sort((a, b) => a.random - b.random)
            .map(({ world }) => world);
        level.checkTermination = [
            CodeCheckers.acceptUserAbort(1),
            CodeCheckers.allowOnlyDone(1),
            function({ worldDef }) {
                if (worldDef == worlds[0])
                    return null;
                return { fine: 1, accept: false, msg: problem ? problem : 'Die gewählte Roboterwelt ist unnötig groß' };
            },
            CodeCheckers.accept(true)
        ];
        return level;
    });
};

window.roboterCodingTask = {
    levels: LEVELS
};
})(window);
